@@ -51,11 +51,11 @@ static void g_icon_class_init (gpointer g_class,
 GType
 g_icon_get_type (void)
 {
-  static GType icon_type = 0;
+  static volatile gsize g_define_type_id__volatile = 0;
 
-  if (! icon_type)
+  if (g_once_init_enter (&g_define_type_id__volatile))
     {
-      static const GTypeInfo icon_info =
+      const GTypeInfo icon_info =
       {
         sizeof (GIconIface), /* class_size */
 	g_icon_base_init,   /* base_init */
@@ -67,15 +67,16 @@ g_icon_get_type (void)
 	0,              /* n_preallocs */
 	NULL
       };
-
-      icon_type =
+      GType g_define_type_id =
 	g_type_register_static (G_TYPE_INTERFACE, I_("GIcon"),
 				&icon_info, 0);
 
-      g_type_interface_add_prerequisite (icon_type, G_TYPE_OBJECT);
+      g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_OBJECT);
+
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
     }
 
-  return icon_type;
+  return g_define_type_id__volatile;
 }
 
 static void
