@@ -29,7 +29,7 @@
 /**
  * SECTION:gseekable
  * @short_description: Stream seeking interface
- * @include: gio.h
+ * @include: gio/gio.h
  * @see_also: #GInputStream, #GOutputStream
  * 
  * #GSeekable is implemented by streams (implementations of 
@@ -44,11 +44,11 @@ static void g_seekable_base_init (gpointer g_class);
 GType
 g_seekable_get_type (void)
 {
-  static GType seekable_type = 0;
+  static volatile gsize g_define_type_id__volatile = 0;
 
-  if (!seekable_type)
+  if (g_once_init_enter (&g_define_type_id__volatile))
     {
-      static const GTypeInfo seekable_info =
+      const GTypeInfo seekable_info =
       {
         sizeof (GSeekableIface), /* class_size */
 	g_seekable_base_init,   /* base_init */
@@ -60,15 +60,16 @@ g_seekable_get_type (void)
 	0,              /* n_preallocs */
 	NULL
       };
-
-      seekable_type =
+      GType g_define_type_id =
 	g_type_register_static (G_TYPE_INTERFACE, I_("GSeekable"),
 				&seekable_info, 0);
 
-      g_type_interface_add_prerequisite (seekable_type, G_TYPE_OBJECT);
+      g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_OBJECT);
+
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
     }
 
-  return seekable_type;
+  return g_define_type_id__volatile;
 }
 
 static void
