@@ -78,11 +78,11 @@ static void g_volume_class_init (gpointer g_class,
 GType
 g_volume_get_type (void)
 {
-  static GType volume_type = 0;
+  static volatile gsize g_define_type_id__volatile = 0;
 
-  if (! volume_type)
+  if (g_once_init_enter (&g_define_type_id__volatile))
     {
-      static const GTypeInfo volume_info =
+      const GTypeInfo volume_info =
       {
         sizeof (GVolumeIface), /* class_size */
 	g_volume_base_init,   /* base_init */
@@ -94,15 +94,16 @@ g_volume_get_type (void)
 	0,              /* n_preallocs */
 	NULL
       };
-
-      volume_type =
+      GType g_define_type_id =
 	g_type_register_static (G_TYPE_INTERFACE, I_("GVolume"),
 				&volume_info, 0);
 
-      g_type_interface_add_prerequisite (volume_type, G_TYPE_OBJECT);
+      g_type_interface_add_prerequisite (g_define_type_id, G_TYPE_OBJECT);
+
+      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
     }
 
-  return volume_type;
+  return g_define_type_id__volatile;
 }
 
 static void
